import os
import uuid
import requests
import functools

from flask import Flask, request, jsonify, abort
from werkzeug.exceptions import HTTPException
from httplib import BAD_REQUEST, UNAUTHORIZED

from nicevirus.scanner import Scanner


app = Flask(__name__)
app.config.from_envvar('CONFIG')

app.scanner = Scanner(app.config['BACKENDS'])


def authed(fn):
    @functools.wraps(fn)
    def wrapper(*args, **kwargs):
        if request.headers.get('api-key') not in app.config['API_KEY']:
            abort(UNAUTHORIZED, 'Invalid API Key')

        return fn(*args, **kwargs)

    return wrapper


@app.route('/')
def index():
    return app.send_static_file('index.html')


@app.route('/scan/url', methods=['POST'])
@authed
def scan_url():
    data = request.get_json() or request.values

    if 'url' not in data:
        abort(BAD_REQUEST, 'URL required')

    tmp_path = '/tmp/{}'.format(uuid.uuid4())

    try:
        r = requests.get(data['url'], stream=True, verify=False)

        size = int(r.headers['Content-Length'])
        if size > app.config['MAX_FILE_SIZE']:
            abort(BAD_REQUEST, 'File size is too large ({} larger than {})'.format(size, app.config['MAX_FILE_SIZE']))

        with open(tmp_path, 'w') as f:
            for chunk in r.iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)

        results = app.scanner.scan(tmp_path)
    except Exception as e:
        if isinstance(e, HTTPException):
            raise e
        abort(BAD_REQUEST, 'Failed to download file: {}'.format(e))
    finally:
        if os.path.exists(tmp_path):
            os.remove(tmp_path)

    if request.values.get('short') == '1':
        return jsonify({'result': any(map(lambda i: i['found'], results.values()))})

    return jsonify(results)


@app.route('/scan/file', methods=['POST'])
@authed
def scan_file():
    pass


@app.route('/health')
def health():
    check = app.scanner.check()
    if not all(check.values()):
        return jsonify(check), 500
    return jsonify(check), 200
